@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # A piano roll viewer/editor
-# Copyright (C) 2012-2015 Filipe Coelho <falktx@falktx.com>
+# Copyright (C) 2012-2020 Filipe Coelho <falktx@falktx.com>
 # Copyright (C) 2014-2015 Perry Nguyen
 #
 # This program is free software; you can redistribute it and/or
@@ -17,26 +17,14 @@
 #
 # For a full copy of the GNU General Public License see the doc/GPL.txt file.
 
-# ------------------------------------------------------------------------------------------------------------
-# Imports (Config)
-
-from carla_config import *
-
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Global)
 
-if config_UseQt5:
-    from PyQt5.QtCore import Qt, QRectF, QPointF, pyqtSignal
-    from PyQt5.QtGui import QColor, QFont, QPen, QPainter
-    from PyQt5.QtWidgets import QGraphicsItem, QGraphicsLineItem, QGraphicsOpacityEffect, QGraphicsRectItem, QGraphicsSimpleTextItem
-    from PyQt5.QtWidgets import QGraphicsScene, QGraphicsView
-    from PyQt5.QtWidgets import QWidget, QLabel, QComboBox, QHBoxLayout, QVBoxLayout, QStyle
-else:
-    from PyQt4.QtCore import Qt, QRectF, QPointF, pyqtSignal
-    from PyQt4.QtGui import QColor, QFont, QPen, QPainter
-    from PyQt4.QtGui import QGraphicsItem, QGraphicsLineItem, QGraphicsOpacityEffect, QGraphicsRectItem, QGraphicsSimpleTextItem
-    from PyQt4.QtGui import QGraphicsScene, QGraphicsView
-    from PyQt4.QtGui import QWidget, QLabel, QComboBox, QSlider, QHBoxLayout, QVBoxLayout, QStyle
+from PyQt5.QtCore import Qt, QRectF, QPointF, pyqtSignal
+from PyQt5.QtGui import QColor, QFont, QPen, QPainter, QTransform
+from PyQt5.QtWidgets import QGraphicsItem, QGraphicsLineItem, QGraphicsOpacityEffect, QGraphicsRectItem, QGraphicsSimpleTextItem
+from PyQt5.QtWidgets import QGraphicsScene, QGraphicsView
+from PyQt5.QtWidgets import QWidget, QLabel, QComboBox, QHBoxLayout, QVBoxLayout, QStyle
 
 # ------------------------------------------------------------------------------------------------------------
 # Imports (Custom)
@@ -258,11 +246,12 @@ class NoteItem(QGraphicsRectItem):
 
 
 class PianoKeyItem(QGraphicsRectItem):
-    def __init__(self, width, height, parent):
+    def __init__(self, width, height, note, parent):
         QGraphicsRectItem.__init__(self, 0, 0, width, height, parent)
         self.setPen(QPen(QColor(0,0,0,80)))
         self.width = width
         self.height = height
+        self.note = note
         self.setFlag(QGraphicsItem.ItemSendsGeometryChanges)
         self.setAcceptHoverEvents(True)
         self.hover_brush = QColor(200, 0, 0)
@@ -297,6 +286,7 @@ class PianoKeyItem(QGraphicsRectItem):
 class PianoRoll(QGraphicsScene):
     '''the piano roll'''
 
+    noteclicked = pyqtSignal(int,bool)
     midievent = pyqtSignal(list)
     measureupdate = pyqtSignal(int)
     modeupdate = pyqtSignal(str)
@@ -307,6 +297,7 @@ class PianoRoll(QGraphicsScene):
         self.mousePos = QPointF()
 
         self.notes = []
+        self.removed_notes = []
         self.selected_notes = []
         self.piano_keys = []
 
@@ -326,8 +317,7 @@ class PianoRoll(QGraphicsScene):
         self.start_octave = -2
         self.end_octave = 8
         self.notes_in_octave = 12
-        self.total_notes = (self.end_octave - self.start_octave) \
-                * self.notes_in_octave + 1
+        self.total_notes = (self.end_octave - self.start_octave) * self.notes_in_octave + 1
         self.piano_height = self.note_height * self.total_notes
         self.octave_height = self.notes_in_octave * self.note_height
 
@@ -344,7 +334,7 @@ class PianoRoll(QGraphicsScene):
         self.quantize_val = quantize_val
 
         ### dummy vars that will be changed
-        self.time_sig = 0
+        self.time_sig = (0,0)
         self.measure_width = 0
         self.num_measures = 0
         self.max_note_length = 0
@@ -354,9 +344,8 @@ class PianoRoll(QGraphicsScene):
         self.piano = None
         self.header = None
         self.play_head = None
+        self.last_piano_note = None
 
-        self.setTimeSig(time_sig)
-        self.setMeasures(num_measures)
         self.setGridDiv()
         self.default_length = 1. / self.grid_div
 
@@ -364,47 +353,38 @@ class PianoRoll(QGraphicsScene):
     # -------------------------------------------------------------------------
     # Callbacks
 
-    def movePlayHead(self, transport_info):
-        # TODO: need conversion between frames and PPQ
-        x = 105. # works for 120bpm
-        total_duration = self.time_sig[0] * self.num_measures * x
-        pos = transport_info['frame'] / x
-        frac = (pos % total_duration) / total_duration
+    def movePlayHead(self, transportInfo):
+        ticksPerBeat = transportInfo['ticksPerBeat']
+        max_ticks = ticksPerBeat * self.time_sig[0] * self.num_measures
+        cur_tick = ticksPerBeat * self.time_sig[0] * transportInfo['bar'] + ticksPerBeat * transportInfo['beat'] + transportInfo['tick']
+        frac = (cur_tick % max_ticks) / max_ticks
         self.play_head.setPos(QPointF(frac * self.grid_width, 0))
 
     def setTimeSig(self, time_sig):
-        try:
-           new_time_sig = list(map(float, time_sig.split('/')))
-           if len(new_time_sig)==2:
-               self.time_sig = new_time_sig
-
-               self.measure_width = self.full_note_width * self.time_sig[0]/self.time_sig[1]
-               self.max_note_length = self.num_measures * self.time_sig[0]/self.time_sig[1]
-               self.grid_width = self.measure_width * self.num_measures
-               self.setGridDiv()
-        except ValueError:
-            pass
+        self.time_sig = time_sig
+        self.measure_width = self.full_note_width * self.time_sig[0]/self.time_sig[1]
+        self.max_note_length = self.num_measures * self.time_sig[0]/self.time_sig[1]
+        self.grid_width = self.measure_width * self.num_measures
+        self.setGridDiv()
 
     def setMeasures(self, measures):
-        try:
-            self.num_measures = float(measures)
-            self.max_note_length = self.num_measures * self.time_sig[0]/self.time_sig[1]
-            self.grid_width = self.measure_width * self.num_measures
-            self.refreshScene()
-        except:
-            pass
+        #try:
+        self.num_measures = float(measures)
+        self.max_note_length = self.num_measures * self.time_sig[0]/self.time_sig[1]
+        self.grid_width = self.measure_width * self.num_measures
+        self.refreshScene()
+        #except:
+            #pass
 
     def setDefaultLength(self, length):
-        try:
-            v = list(map(float, length.split('/')))
-            if len(v) < 3:
-                self.default_length = \
-                        v[0] if len(v)==1 else \
-                        v[0] / v[1]
-                pos = self.enforce_bounds(self.mousePos)
-                if self.insert_mode: self.makeGhostNote(pos.x(), pos.y())
-        except ValueError:
-            pass
+        v = list(map(float, length.split('/')))
+        if len(v) < 3:
+            self.default_length = \
+                    v[0] if len(v)==1 else \
+                    v[0] / v[1]
+            pos = self.enforce_bounds(self.mousePos)
+            if self.insert_mode:
+                self.makeGhostNote(pos.x(), pos.y())
 
     def setGridDiv(self, div=None):
         if not div: div = self.quantize_val
@@ -421,16 +401,13 @@ class PianoRoll(QGraphicsScene):
             pass
 
     def setQuantize(self, value):
-        try:
-            val = list(map(float, value.split('/')))
-            if len(val) == 1:
-                self.quantize(val[0])
-                self.quantize_val = value
-            elif len(val) == 2:
-                self.quantize(val[0] / val[1])
-                self.quantize_val = value
-        except ValueError:
-            pass
+        val = list(map(float, value.split('/')))
+        if len(val) == 1:
+            self.quantize(val[0])
+            self.quantize_val = value
+        elif len(val) == 2:
+            self.quantize(val[0] / val[1])
+            self.quantize_val = value
 
     # -------------------------------------------------------------------------
     # Event Callbacks
@@ -480,6 +457,14 @@ class PianoRoll(QGraphicsScene):
 
     def mousePressEvent(self, event):
         QGraphicsScene.mousePressEvent(self, event)
+
+        if self.piano.contains(event.scenePos()):
+            item = self.itemAt(event.scenePos(), QTransform())
+            if isinstance(item, PianoKeyItem) and item.note is not None:
+                self.last_piano_note = item.note
+                self.noteclicked.emit(self.last_piano_note, True)
+                return
+
         if not (any(key.pressed for key in self.piano_keys)
                 or any(note.pressed for note in self.notes)):
             for note in self.selected_notes:
@@ -495,6 +480,7 @@ class PianoRoll(QGraphicsScene):
                     self.marquee = QGraphicsRectItem(self.marquee_rect)
                     self.marquee.setBrush(QColor(255, 255, 255, 100))
                     self.addItem(self.marquee)
+
         else:
             for s_note in self.notes:
                 if s_note.pressed and s_note in self.selected_notes:
@@ -510,6 +496,10 @@ class PianoRoll(QGraphicsScene):
 
     def mouseMoveEvent(self, event):
         QGraphicsScene.mouseMoveEvent(self, event)
+
+        if self.last_piano_note is not None:
+            return
+
         self.mousePos = event.scenePos()
         if not (any((key.pressed for key in self.piano_keys))):
             m_pos = event.scenePos()
@@ -570,6 +560,13 @@ class PianoRoll(QGraphicsScene):
                             note.moveEvent(event)
 
     def mouseReleaseEvent(self, event):
+        QGraphicsScene.mouseReleaseEvent(self, event)
+
+        if self.last_piano_note is not None:
+            self.noteclicked.emit(self.last_piano_note, False)
+            self.last_piano_note = None
+            return
+
         if not (any((key.pressed for key in self.piano_keys)) or any((note.pressed for note in self.notes))):
             if event.button() == Qt.LeftButton:
                 if self.place_ghost and self.insert_mode:
@@ -583,6 +580,7 @@ class PianoRoll(QGraphicsScene):
                 elif self.marquee_select:
                     self.marquee_select = False
                     self.removeItem(self.marquee)
+
         elif not self.marquee_select:
             for note in self.selected_notes:
                 old_info = note.note[:]
@@ -612,32 +610,35 @@ class PianoRoll(QGraphicsScene):
         self.piano.setPos(0, self.header_height)
         self.addItem(self.piano)
 
-        key = PianoKeyItem(piano_keys_width, self.note_height, self.piano)
-        label = QGraphicsSimpleTextItem('C8', key)
+        key = PianoKeyItem(piano_keys_width, self.note_height, 78, self.piano)
+        label = QGraphicsSimpleTextItem('C9', key)
         label.setPos(18, 1)
         label.setFont(piano_label)
         key.setBrush(QColor(255, 255, 255))
-        for i in range(self.end_octave - self.start_octave, self.start_octave - self.start_octave, -1):
+        for i in range(self.end_octave - self.start_octave, 0, -1):
             for j in range(self.notes_in_octave, 0, -1):
+                note = (self.end_octave - i + 3) * 12 - j
                 if j in black_notes:
-                    key = PianoKeyItem(piano_keys_width/1.4, self.note_height, self.piano)
+                    key = PianoKeyItem(piano_keys_width/1.4, self.note_height, note, self.piano)
                     key.setBrush(QColor(0, 0, 0))
                     key.setZValue(1.0)
                     key.setPos(0, self.note_height * j + self.octave_height * (i - 1))
                 elif (j - 1) and (j + 1) in black_notes:
-                    key = PianoKeyItem(piano_keys_width, self.note_height * 2, self.piano)
+                    key = PianoKeyItem(piano_keys_width, self.note_height * 2, note, self.piano)
                     key.setBrush(QColor(255, 255, 255))
                     key.setPos(0, self.note_height * j + self.octave_height * (i - 1) - self.note_height/2.)
                 elif (j - 1) in black_notes:
-                    key = PianoKeyItem(piano_keys_width, self.note_height * 3./2, self.piano)
+                    key = PianoKeyItem(piano_keys_width, self.note_height * 3./2, note, self.piano)
                     key.setBrush(QColor(255, 255, 255))
                     key.setPos(0, self.note_height * j + self.octave_height * (i - 1) - self.note_height/2.)
                 elif (j + 1) in black_notes:
-                    key = PianoKeyItem(piano_keys_width, self.note_height * 3./2, self.piano)
+                    key = PianoKeyItem(piano_keys_width, self.note_height * 3./2, note, self.piano)
                     key.setBrush(QColor(255, 255, 255))
                     key.setPos(0, self.note_height * j + self.octave_height * (i - 1))
                 if j == 12:
-                    label = QGraphicsSimpleTextItem('{}{}'.format(labels[j - 1], self.end_octave - i), key )
+                    label = QGraphicsSimpleTextItem('{}{}'.format(labels[j - 1],
+                                                                  self.end_octave - i + 1),
+                                                                  key)
                     label.setPos(18, 6)
                     label.setFont(piano_label)
                 self.piano_keys.append(key)
@@ -693,10 +694,12 @@ class PianoRoll(QGraphicsScene):
         self.drawHeader()
         self.drawGrid()
         self.drawPlayHead()
+
         for note in self.notes[:]:
             if note.note[1] >= (self.num_measures * self.time_sig[0]):
                 self.notes.remove(note)
-                self.midievent.emit(["midievent-remove", note.note[0], note.note[1], note.note[2], note.note[3]])
+                self.removed_notes.append(note)
+                #self.midievent.emit(["midievent-remove", note.note[0], note.note[1], note.note[2], note.note[3]])
             elif note.note[2] > self.max_note_length:
                 new_note = note.note[:]
                 new_note[2] = self.max_note_length
@@ -704,6 +707,12 @@ class PianoRoll(QGraphicsScene):
                 self.drawNote(new_note[0], new_note[1], self.max_note_length, new_note[3], False)
                 self.midievent.emit(["midievent-remove", note.note[0], note.note[1], note.note[2], note.note[3]])
                 self.midievent.emit(["midievent-add", new_note[0], new_note[1], new_note[2], new_note[3]])
+
+        for note in self.removed_notes[:]:
+            if note.note[1] < (self.num_measures * self.time_sig[0]):
+                self.removed_notes.remove(note)
+                self.notes.append(note)
+
         list(map(self.addItem, self.notes))
         if self.views():
             self.views()[0].setSceneRect(self.itemsBoundingRect())
@@ -711,6 +720,7 @@ class PianoRoll(QGraphicsScene):
     def clearNotes(self):
         self.clear()
         self.notes = []
+        self.removed_notes = []
         self.selected_notes = []
         self.drawPiano()
         self.drawHeader()
@@ -774,10 +784,10 @@ class PianoRoll(QGraphicsScene):
         if self.snap_value:
             pos_x = int(round((pos_x - self.piano_width) / self.snap_value)) \
                     * self.snap_value + self.piano_width
-        if pos_y:
+        if pos_y is not None:
             pos_y = int((pos_y - self.header_height) / self.note_height) \
                     * self.note_height + self.header_height
-        return (pos_x, pos_y) if pos_y else pos_x
+        return (pos_x, pos_y) if pos_y is not None else pos_x
 
     def adjust_note_vel(self, event):
         m_pos = event.scenePos()
@@ -863,10 +873,9 @@ class ModeIndicator(QWidget):
         self.mode = None
 
     def paintEvent(self, event):
-        painter = QPainter(self)
         event.accept()
 
-        painter.begin(self)
+        painter = QPainter(self)
 
         painter.setPen(QPen(QColor(0, 0, 0, 0)))
         if self.mode == 'velocity_mode':
@@ -877,9 +886,6 @@ class ModeIndicator(QWidget):
             painter.setBrush(QColor(0, 0, 0, 0))
         painter.drawRect(0, 0, 30, 20)
 
-        # FIXME
-        painter.end()
-
     def changeMode(self, new_mode):
         self.mode = new_mode
         self.update()
